use super::{VerbContext, UserVerb, UserVerbRef, UResult,
            user_error, get_player_item_or_fail};
use crate::{
    models::item::{StatType, SkillType}
};
use async_trait::async_trait;
use ansi::ansi;

pub struct Verb;
#[async_trait]
impl UserVerb for Verb {
    async fn handle(self: &Self, ctx: &mut VerbContext, _verb: &str, _remaining: &str) -> UResult<()> {
        let player_item = get_player_item_or_fail(ctx).await?;

        let user = match ctx.user_dat {
            None => user_error("Log in first".to_owned())?,
            Some(user) => user
        };

        let mut msg = String::new();
        msg.push_str(&format!(ansi!("<bgblue><white><bold>| {:11} | {:5} | {:5} |<reset>\n"),
                             "Stat", "Raw", "Total"
        ));
        for st in StatType::values().iter() {
            msg.push_str(&format!(ansi!("| <bold>{:11}<reset> | {:5.2} | {:5.2} |\n"),
                                  st.display(),
                                  user.raw_stats.get(st).unwrap_or(&0.0),
                                  player_item.total_stats.get(st).unwrap_or(&0.0)
            ));
        }
        msg.push_str("\n");

        msg.push_str(&format!(ansi!("<bgblue><white><bold>| {:11} | {:5} | {:5} |<reset>\n"),
                             "Skill", "Raw", "Total"
        ));
        for st in SkillType::values().iter() {
            msg.push_str(&format!(ansi!("| <bold>{:11}<reset> | {:5.2} | {:5.2} |\n"),
                                  st.display(),
                                  user.raw_skills.get(st).unwrap_or(&0.0),
                                  player_item.total_skills.get(st).unwrap_or(&0.0)
            ));
        }
        msg.push_str("\n");
        msg.push_str(&format!(ansi!("Experience: <bold>{}<reset> total, \
                                     change of {} this re-roll, \
                                     {} spent since re-roll.\n"),
                             player_item.total_xp,
                             user.experience.xp_change_for_this_reroll,
                             user.experience.spent_xp
        ));
        
        ctx.trans.queue_for_session(ctx.session, Some(&msg)).await?;
        
        Ok(())
    }
}
static VERB_INT: Verb = Verb;
pub static VERB: UserVerbRef = &VERB_INT as UserVerbRef;
