use super::{VerbContext, UserVerb, UserVerbRef, UResult, user_error, get_player_item_or_fail, search_item_for_user};
use crate::{
    db::ItemSearchParams,
    static_content::possession_type::possession_data,
};
use async_trait::async_trait;

pub struct Verb;
#[async_trait]
impl UserVerb for Verb {
    async fn handle(self: &Self, ctx: &mut VerbContext, _verb: &str, remaining: &str) -> UResult<()> {
        let (write_what_raw, on_what_raw) = match remaining.rsplit_once(" on ") {
            None => user_error("Write on what? Try write something on something".to_owned())?,
            Some(v) => v
        };
        let player_item = get_player_item_or_fail(ctx).await?;
        let item = search_item_for_user(ctx, &ItemSearchParams {
            include_contents: true,
            ..ItemSearchParams::base(&player_item, on_what_raw.trim())
        }).await?;
        if item.item_type != "possession" {
            user_error("You can't write on that!".to_owned())?;
        }
        let handler = match item.possession_type.as_ref()
            .and_then(|pt| possession_data().get(pt))
            .and_then(|pd| pd.write_handler) {
            None => user_error("You can't write on that!".to_owned())?,
            Some(h) => h
        };

        handler.write_cmd(ctx, &player_item, &item, write_what_raw.trim()).await?;
        
        Ok(())
    }
}
static VERB_INT: Verb = Verb;
pub static VERB: UserVerbRef = &VERB_INT as UserVerbRef;
