use super::{PossessionData, UseData, UseEffect, ChargeData};
use crate::models::{
    item::SkillType,
    consent::ConsentType,
};

use super::PossessionType::*;

pub fn medium_data() -> PossessionData {
  PossessionData {
      display: "medium trauma kit",
      details: "A collection of bandages and and small gadgets that look like they could, in the right hands, make an unhealthy person healthy again. It looks like when brand new, it could be used 5 times.",
      aliases: vec!("trauma"),
      charge_data: Some(ChargeData {
          max_charges: 5,
          charge_name_prefix: "treatment",
          charge_name_suffix: "worth of supplies",
          ..Default::default()
      }),
      use_data: Some(UseData {
          uses_skill: SkillType::Medic,
          diff_level: 10.0,
          crit_fail_effects: vec!(
              UseEffect::BroadcastMessage {
                  messagef: Box::new(|player, _item, target| (
                      format!(
                          "{} attempts to heal {} with a trauma kit, but fucks it up badly\n",
                          &player.display_for_sentence(true, 1, true),
                          &if target.item_type == player.item_type && target.item_code == player.item_code {
                              player.pronouns.intensive.clone()
                          } else {
                              target.display_for_sentence(true, 1, false)
                          }
                      ),
                      format!("{} attempts to heal {} with a trauma kit, but messes it up badly\n",
                              &player.display_for_sentence(false, 1, true),
                              &if target.item_type == player.item_type && target.item_code == player.item_code {
                                  player.pronouns.intensive.clone()
                              } else {
                                  target.display_for_sentence(false, 1, false)
                              }
                      )))
              },
              UseEffect::ChangeTargetHealth {
                  delay_secs: 0, base_effect: -2, skill_multiplier: -3.0,
                  max_effect: -5,
                  message: Box::new(
                      |target|
                      (format!(
                          "Fuck! The trauma kit makes {}'s condition worse",
                          target.display_for_sentence(true, 1, false)),
                       format!(
                           "The trauma kit makes {}'s condition worse",
                           target.display_for_sentence(false, 1, false)
                       )
                      ))
              }
          ),
          fail_effects: vec!(
              UseEffect::BroadcastMessage {
                  messagef: Box::new(|player, _item, target| (
                      format!(
                          "{} attempts unsuccessfully to heal {} with a trauma kit\n",
                          &player.display_for_sentence(true, 1, true),
                          &if target.item_type == player.item_type && target.item_code == player.item_code {
                              player.pronouns.intensive.clone()
                          } else {
                              target.display_for_sentence(true, 1, false)
                          }
                      ),
                      format!("{} attempts unsuccessfully to heal {} with a trauma kit\n",
                              &player.display_for_sentence(false, 1, true),
                              &if target.item_type == player.item_type && target.item_code == player.item_code {
                                  player.pronouns.intensive.clone()
                              } else {
                                  target.display_for_sentence(false, 1, false)
                              }
                      )))
              },
          ),
          success_effects: vec!(
              UseEffect::BroadcastMessage {
                  messagef: Box::new(|player, _item, target| (
                      format!(
                          "{} expertly heals {} with a trauma kit\n",
                          &player.display_for_sentence(true, 1, true),
                          &if target.item_type == player.item_type && target.item_code == player.item_code {
                              player.pronouns.intensive.clone()
                          } else {
                              target.display_for_sentence(true, 1, false)
                          }
                      ),
                      format!("{} expertly heals {} with a trauma kit\n",
                              &player.display_for_sentence(false, 1, true),
                              &if target.item_type == player.item_type && target.item_code == player.item_code {
                                  player.pronouns.intensive.clone()
                              } else {
                                  target.display_for_sentence(false, 1, false)
                              }
                      )))
              },
              UseEffect::ChangeTargetHealth {
                  delay_secs: 0, base_effect: 2, skill_multiplier: 8.0,
                  max_effect: 10,
                  message: Box::new(
                      |target|
                      (format!(
                          "FUUUCK! It hurts {}, but also starts to soothe {}",
                          target.display_for_sentence(true, 1, false),
                          &target.pronouns.object
                      ),
                       format!(
                           "It hurts {}, but also starts to soothe {}",
                           target.display_for_sentence(true, 1, false),
                           &target.pronouns.object
                       ))
                  )
              },
              UseEffect::ChangeTargetHealth {
                  delay_secs: 10, base_effect: 2, skill_multiplier: 7.0,
                  max_effect: 9,
                  message: Box::new(
                      |target|
                      (format!(
                          "FUUUCK! It hurts {}, but also starts to soothe {}",
                          target.display_for_sentence(true, 1, false),
                          &target.pronouns.object
                      ),
                       format!(
                           "It hurts {}, but also starts to soothe {}",
                           target.display_for_sentence(true, 1, false),
                           &target.pronouns.object
                       ))
                  )
              },
              UseEffect::ChangeTargetHealth {
                  delay_secs: 20, base_effect: 1, skill_multiplier: 6.0,
                  max_effect: 7,
                  message: Box::new(
                      |target|
                      (format!(
                          "The bandages soothe {}'s wounds",
                          target.display_for_sentence(true, 1, false),
                      ),
                       format!(
                           "The bandages soothe {}'s wounds",
                           target.display_for_sentence(false, 1, false),
                       ))
                  )
              },
              UseEffect::ChangeTargetHealth {
                  delay_secs: 30, base_effect: 1, skill_multiplier: 5.0,
                  max_effect: 6,
                  message: Box::new(
                      |target|
                      (format!(
                          "The bandages soothe {}'s wounds",
                          target.display_for_sentence(true, 1, false),
                      ),
                       format!(
                           "The bandages soothe {}'s wounds",
                           target.display_for_sentence(false, 1, false),
                       ))
                  )
              },
              UseEffect::ChangeTargetHealth {
                  delay_secs: 40, base_effect: 0, skill_multiplier: 4.0,
                  max_effect: 4,
                  message: Box::new(
                      |target|
                      (format!(
                          "The bandages soothe {}'s wounds",
                          target.display_for_sentence(true, 1, false),
                      ),
                       format!(
                           "The bandages soothe {}'s wounds",
                           target.display_for_sentence(false, 1, false),
                       ))
                  )
              },
              UseEffect::ChangeTargetHealth {
                  delay_secs: 50, base_effect: 0, skill_multiplier: 3.0,
                  max_effect: 3,
                  message: Box::new(
                      |target|
                      (format!(
                          "The bandages soothe {}'s wounds",
                          target.display_for_sentence(true, 1, false),
                      ),
                       format!(
                           "The bandages soothe {}'s wounds",
                           target.display_for_sentence(false, 1, false),
                       ))
                  )
              },
              UseEffect::ChangeTargetHealth {
                  delay_secs: 60, base_effect: 0, skill_multiplier: 2.0,
                  max_effect: 2,
                  message: Box::new(
                      |target|
                      (format!(
                          "The bandages soothe {}'s wounds",
                          target.display_for_sentence(true, 1, false),
                      ),
                       format!(
                           "The bandages soothe {}'s wounds",
                           target.display_for_sentence(false, 1, false),
                       ))
                  )
              },
          ),
          task_ref: "bandage",
          errorf: Box::new(
              |_item, target|
              if target.is_dead {
                  Some(format!("It is too late, {}'s dead", target.pronouns.subject))
              } else if target.item_type != "player" && target.item_type != "npc" {
                  Some("It only works on animals.".to_owned())
              } else {
                  None
              }),
          needs_consent_check: Some(ConsentType::Medicine),
          ..Default::default()
      }),
      becomes_on_spent: Some(EmptyMedicalBox),
      ..Default::default()
  }
}

pub fn empty_data() -> PossessionData { 
    PossessionData {
        display: "empty medical box",
        details: "An empty box that looks like it once had something medical in it.",
        aliases: vec!("box"),
        ..Default::default()
    }   
}
